/* BdfServer - Copyright (c) 2012-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.impl;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.L10nManager;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.FicheBlocks;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.format.FormatContext;
import net.fichotheque.format.FormatSource;
import net.fichotheque.format.formatters.ExtractionFormatter;
import net.fichotheque.format.formatters.FicheBlockFormatter;
import net.fichotheque.include.LiageTest;
import net.fichotheque.tools.format.formatters.ExtractionFormatterParser;
import net.fichotheque.xml.extraction.ExtractParameters;
import net.fichotheque.xml.extraction.ExtractionXMLUtils;
import net.mapeadores.util.instruction.InstructionResolverProvider;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.MessageLocalisationProvider;


/**
 *
 * @author Vincent Calame
 */
class FormatContextImpl implements FormatContext {

    private final static ParamResolver EMPTY_PARAMMAP = new ParamResolver();
    private final BdfServer bdfServer;
    private final InstructionResolverProvider instructionResolverProvider;
    private final TransformationManager transformationManager;
    private final L10nManager l10nManager;
    private final UiManager uiManager;

    FormatContextImpl(BdfServer bdfServer, InstructionResolverProvider instructionResolverProvider, TransformationManager transformationManager, L10nManager l10nManager, UiManager uiManager) {
        this.bdfServer = bdfServer;
        this.instructionResolverProvider = instructionResolverProvider;
        this.transformationManager = transformationManager;
        this.l10nManager = l10nManager;
        this.uiManager = uiManager;
    }

    @Override
    public Fichotheque getFichotheque() {
        return bdfServer.getFichotheque();
    }

    @Override
    public InstructionResolverProvider getInstructionResolverProvider() {
        return instructionResolverProvider;
    }

    @Override
    public ExtractionFormatter getExtractionFormatter(String name, Map<String, Object> options) {
        if ((name == null) || (name.equals(FormatContext.XML_SPECIAL_FORMATTER_NAME))) {
            return ExtractionFormatterParser.XML_PART;
        }
        try {
            TemplateKey templateKey = TemplateKey.parse(TransformationKey.FORMAT_TRANSFORMATIONKEY, name);
            SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey, false);
            if (simpleTemplate != null) {
                return new SimpleTemplateExtractionFormatter(simpleTemplate, convertToParamResolver(options));
            } else {
                return null;
            }
        } catch (ParseException pe) {
            return null;
        }
    }

    @Override
    public FicheBlockFormatter getFicheBlockFormatter(String name, Map<String, Object> options) {
        if ((name != null) && (name.equals(FormatContext.XML_SPECIAL_FORMATTER_NAME))) {
            return new XmlFicheBlockFormatter(ExtractionConstants.INITIAL_VERSION, false);
        }
        try {
            TemplateKey templateKey = TemplateKey.parse(TransformationKey.SECTION_TRANSFORMATIONKEY, name);
            SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey, false);
            if (simpleTemplate != null) {
                return new SimpleTemplateFicheBlockFormatter(simpleTemplate, convertToParamResolver(options));
            } else {
                return null;
            }
        } catch (ParseException pe) {
            return null;
        }
    }

    @Override
    public LiageTest getLiageTest(Corpus corpus) {
        return UiUtils.checkLiageTest(uiManager.getMainUiComponents(corpus));
    }

    @Override
    public MessageLocalisationProvider getMessageLocalisationProvider() {
        return l10nManager;
    }

    @Override
    public Object getContextObject(String objectName) {
        return bdfServer.getContextObject(objectName);
    }

    private static ParamResolver convertToParamResolver(Map<String, Object> options) {
        if (options.isEmpty()) {
            return EMPTY_PARAMMAP;
        }
        ParamResolver paramResolver = new ParamResolver();
        paramResolver.convertOptions(options);
        return paramResolver;
    }


    private static class SimpleTemplateExtractionFormatter implements ExtractionFormatter {

        private final SimpleTemplate simpleTemplate;
        private final ParamResolver paramResolver;

        private SimpleTemplateExtractionFormatter(SimpleTemplate simpleTemplate, ParamResolver paramWrapper) {
            this.simpleTemplate = simpleTemplate;
            this.paramResolver = paramWrapper;
        }

        @Override
        public String formatExtraction(String extractionString, FormatSource formatSource) {
            return simpleTemplate.transform(extractionString, paramResolver.toTranformParamMap(formatSource));
        }

    }


    private static class SimpleTemplateFicheBlockFormatter implements FicheBlockFormatter {

        private final SimpleTemplate simpleTemplate;
        private final ParamResolver paramResolver;

        private SimpleTemplateFicheBlockFormatter(SimpleTemplate simpleTemplate, ParamResolver paramWrapper) {
            this.simpleTemplate = simpleTemplate;
            this.paramResolver = paramWrapper;
        }

        @Override
        public String formatFicheBlocks(FicheBlocks ficheBlocks, FormatSource formatSource, SubsetKey defaultCorpusKey) {
            ExtractParameters extractParameters = BdfTransformationUtils.buildExtractParameters(formatSource.getExtractionContext(), simpleTemplate.getAttributes(), null);
            String extractionString = ExtractionXMLUtils.ficheBlocksToExtractionString(ficheBlocks, extractParameters, defaultCorpusKey);
            return simpleTemplate.transform(extractionString, paramResolver.toTranformParamMap(formatSource));
        }

    }


    private static class XmlFicheBlockFormatter implements FicheBlockFormatter {

        private final int extractVersion;
        private final boolean withEmpty;

        private XmlFicheBlockFormatter(int extractVersion, boolean withEmpty) {
            this.extractVersion = extractVersion;
            this.withEmpty = withEmpty;
        }

        @Override
        public String formatFicheBlocks(FicheBlocks ficheBlocks, FormatSource formatSource, SubsetKey defaultCorpusKey) {
            return ExtractionXMLUtils.ficheBlocksToExtractionString(ficheBlocks, new ExtractParameters(formatSource.getExtractionContext(), extractVersion, withEmpty), defaultCorpusKey);
        }

    }


    private static class ParamResolver {

        private final Map<String, Object> otherParamMap = new HashMap<String, Object>();
        private boolean ficheLang = false;
        private Lang lang = null;

        private ParamResolver() {

        }

        private void convertOptions(Map<String, Object> options) {
            for (Map.Entry<String, Object> entry : options.entrySet()) {
                Object value = entry.getValue();
                switch (entry.getKey()) {
                    case "startlevel":
                        if (value instanceof Integer) {
                            otherParamMap.put(TransformationConstants.STARTLEVEL_PARAMETER, value);
                        }
                        break;
                    case "externaltarget":
                        otherParamMap.put(TransformationConstants.EXTERNALTARGET_PARAMETER, value);
                        break;
                    case "lang":
                        if (value.equals("_fiche")) {
                            ficheLang = true;
                            lang = null;
                        } else {
                            try {
                                lang = Lang.parse(value.toString());
                                ficheLang = false;
                            } catch (ParseException pe) {
                                lang = null;
                                ficheLang = false;
                            }
                        }
                        break;
                }
            }
        }

        private Map<String, Object> toTranformParamMap(FormatSource formatSource) {
            Lang workingLang = null;
            if (lang != null) {
                workingLang = lang;
            } else if (ficheLang) {
                SubsetItem currentSubsetItem = formatSource.getSubsetItemPointeur().getCurrentSubsetItem();
                if ((currentSubsetItem != null) && (currentSubsetItem instanceof FicheMeta)) {
                    workingLang = ((FicheMeta) currentSubsetItem).getLang();
                }
            }
            if (workingLang == null) {
                workingLang = LocalisationUtils.getDefaultLang(formatSource.getLangContext());
            }
            Map<String, Object> transformParamMap = BdfTransformationUtils.newParamMap(workingLang);
            transformParamMap.putAll(otherParamMap);
            return transformParamMap;
        }

    }

}
