/* BdfServer - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.exportation.transformation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.configuration.LangConfiguration;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.namespaces.BdfUserSpace;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.SpecialIncludeUi;
import fr.exemole.bdfserver.api.ui.SubsetIncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.BdfURI;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.TemplateCompiler;
import fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates.DefaultOptions;
import fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates.TemplateStorageUnitFactory;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.function.Predicate;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.TemplateContentDescription;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.include.LiageTest;
import net.fichotheque.namespaces.TransformationSpace;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.exportation.transformation.TransformationCheck;
import net.fichotheque.tools.extraction.ExtractionEngine;
import net.fichotheque.tools.extraction.builders.CorpusExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtractionBuilderUtils;
import net.fichotheque.tools.extraction.builders.ExtractionDefBuilder;
import net.fichotheque.tools.extraction.builders.FicheFilterBuilder;
import net.fichotheque.tools.extraction.builders.FicheRattachementDefBuilder;
import net.fichotheque.tools.extraction.builders.FilterUnitListBuilder;
import net.fichotheque.tools.extraction.builders.MotcleFilterBuilder;
import net.fichotheque.tools.extraction.builders.MotcleRattachementDefBuilder;
import net.fichotheque.tools.extraction.builders.ThesaurusExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.TitleClauseBuilder;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.utils.ExtractionUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.TransformationUtils;
import net.fichotheque.xml.extraction.ExtractParameters;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.attr.AttributesBuilder;
import net.mapeadores.util.exceptions.NestedTransformerException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.MessageByLine;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public final class BdfTransformationUtils {

    public final static Attributes XSLT_DEFAULT_TEMPLATE_ATTRIBUTES = AttributesBuilder.init().appendValue(TransformationSpace.EXTRACTVERSION_KEY, String.valueOf(ExtractionConstants.PLURAL_VERSION)).toAttributes();
    public final static Attributes HTML_XSLT_DEFAULT_TEMPLATE_ATTRIBUTES = AttributesBuilder.init(XSLT_DEFAULT_TEMPLATE_ATTRIBUTES)
            .appendValue(TransformationSpace.EMPTYCOMPONENTS_KEY, "true")
            .toAttributes();
    public final static Attributes ODT_XSLT_DEFAULT_TEMPLATE_ATTRIBUTES = AttributesBuilder.init(XSLT_DEFAULT_TEMPLATE_ATTRIBUTES)
            .appendValue(TransformationSpace.EMPTYCOMPONENTS_KEY, "false")
            .toAttributes();
    private final static TemplateDescription[] EMPTY_ARRAY = new TemplateDescription[0];

    private BdfTransformationUtils() {
    }

    public static TemplateDescription getTemplateDescription(TransformationManager transformationManager, TemplateKey templateKey) {
        TransformationDescription transformationDescription = transformationManager.getTransformationDescription(templateKey.getTransformationKey());
        if (templateKey.isSimpleTemplate()) {
            for (TemplateDescription templateDescription : transformationDescription.getSimpleTemplateDescriptionList()) {
                if (templateDescription.getTemplateKey().equals(templateKey)) {
                    return templateDescription;
                }
            }
        } else {
            for (TemplateDescription templateDescription : transformationDescription.getStreamTemplateDescriptionList()) {
                if (templateDescription.getTemplateKey().equals(templateKey)) {
                    return templateDescription;
                }
            }
        }
        return null;
    }


    public static String getFicheString(BdfParameters bdfParameters, String simpleTemplateName, FicheMeta ficheMeta, Lang lang, String fichothequePath) {
        BdfServer bdfServer = bdfParameters.getBdfServer();
        LangContext langContext = LocalisationUtils.toUserLangContext(lang);
        TransformationKey corpusTransformationKey = new TransformationKey(ficheMeta.getSubsetKey());
        TemplateKey templateKey;
        try {
            templateKey = TemplateKey.parse(corpusTransformationKey, simpleTemplateName);
        } catch (ParseException pe) {
            templateKey = TemplateKey.toDefault(corpusTransformationKey);
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey, true);
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, langContext)
                .setSubsetAccessPredicate(bdfParameters.getPermissionSummary().getSubsetAccessPredicate())
                .toExtractionContext();
        ExtractionDef extractionDef = simpleTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = getDefaultExtractionDef(bdfServer, corpusTransformationKey, simpleTemplate.getAttributes());
        }
        ExtractParameters extractParameters = buildExtractParameters(extractionContext, simpleTemplate.getAttributes(), bdfParameters);
        String extractionString = ExtractionEngine.init(extractParameters, extractionDef).run(ExtractionBuilderUtils.getExtractionSource(ficheMeta, extractionContext, extractionDef));
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(lang);
        paramMap.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, fichothequePath);
        return simpleTemplate.transform(extractionString, paramMap);
    }


    public static Map<String, Object> newParamMap(Lang workingLang) {
        Map<String, Object> map = new HashMap<String, Object>();
        map.put(TransformationConstants.WORKINGLANG_PARAMETER, workingLang.toString());
        return map;
    }

    public static Map<String, Object> newParamMap(BdfParameters bdfParameters) {
        BdfServer bdfServer = bdfParameters.getBdfServer();
        BdfUser bdfUser = bdfParameters.getBdfUser();
        Redacteur redacteur = bdfUser.getRedacteur();
        Map<String, Object> map = new HashMap<String, Object>();
        map.put(TransformationConstants.WORKINGLANG_PARAMETER, bdfUser.getWorkingLang().toString());
        map.put(TransformationConstants.BDF_VERSION_PARAMETER, bdfServer.getBuildInfo().getVersion());
        Attribute attribute = bdfServer.getFichotheque().getFichothequeMetadata().getAttributes().getAttribute(BdfUserSpace.INCLUDESCRIPTS_KEY);
        String includeScripts = "";
        if (attribute != null) {
            int size = attribute.size();
            if (size == 1) {
                includeScripts = attribute.getFirstValue();
            } else {
                StringBuilder buf = new StringBuilder();
                for (String value : attribute) {
                    if (buf.length() > 0) {
                        buf.append(' ');
                    }
                    buf.append(value);
                }
                includeScripts = buf.toString();
            }
        }
        map.put(TransformationConstants.INCLUDESCRIPTS_PARAMETER, includeScripts);
        map.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, BdfUserUtils.getRootUrl(bdfUser));
        boolean hideEmpty = BdfUserUtils.getBoolean(bdfUser, BdfUserSpace.HIDEEMPTY_KEY);
        map.put(TransformationConstants.SHOWEMPTY_PARAMETER, (hideEmpty) ? 0 : 1);
        map.put(TransformationConstants.USER_SPHERE_PARAMETER, redacteur.getSubsetName());
        map.put(TransformationConstants.USER_LOGIN_PARAMETER, redacteur.getLogin());
        return map;
    }

    public static void putFileNameParameter(Map<String, String> paramMap, String fileName) {
        paramMap.put(TransformationConstants.BDF_FILENAME_PARAMETER, fileName);
        String fileBaseName = fileName;
        String fileExtension = "";
        int idx = fileName.lastIndexOf('.');
        if (idx != -1) {
            fileBaseName = fileName.substring(0, idx);
            fileExtension = fileName.substring(idx + 1);
        }
        paramMap.put(TransformationConstants.BDF_FILEBASENAME_PARAMETER, fileBaseName);
        paramMap.put(TransformationConstants.BDF_FILEEXTENSION_PARAMETER, fileExtension);
    }

    public static TemplateStorage.Unit getDefaultUnit(BdfServer bdfServer, TransformationKey transformationKey, String extension) {
        DefaultOptions options = DefaultOptions.init()
                .withExtractionDef(false);
        if (extension == null) {
            return TemplateStorageUnitFactory.newXsltHtml(bdfServer, TemplateKey.toDefault(transformationKey), options, HTML_XSLT_DEFAULT_TEMPLATE_ATTRIBUTES);
        } else if (bdfServer.getTransformationManager().isDefaultStreamTemplateAvailable(transformationKey, extension)) {
            try {
                ValidExtension validExtension = ValidExtension.parse(extension);
                TemplateKey templateKey = TemplateKey.toDefault(transformationKey, validExtension);
                if (extension.equals("odt")) {
                    return TemplateStorageUnitFactory.newXsltOdt(bdfServer, templateKey, options, ODT_XSLT_DEFAULT_TEMPLATE_ATTRIBUTES);
                } else {
                    return null;
                }
            } catch (ParseException pe) {
                return null;
            }
        } else {
            return null;
        }
    }

    public static ExtractionDef getDefaultExtractionDef(BdfServer bdfServer, TransformationKey transformationKey, Attributes attributes) {
        ExtractionDef defaultExtractionDef;
        if (transformationKey.isCorpusTransformationKey()) {
            defaultExtractionDef = getFicheExtractionDef(bdfServer, transformationKey.toCorpusKey(), attributes);
        } else if (transformationKey.equals(TransformationKey.COMPILATION_TRANSFORMATIONKEY)) {
            defaultExtractionDef = getCompilationExtractionDef(bdfServer, attributes);
        } else {
            if (transformationKey.equals(TransformationKey.STATTHESAURUS_TRANSFORMATIONKEY)) {
                MotcleFilter thesaurusFilter = MotcleFilterBuilder.init(MotcleFilter.DEFAULT_TYPE)
                        .setWithLabels(true)
                        .addDefaultIndexationCorpusExtractDef()
                        .setChildrenRecursive()
                        .toMotcleFilter();
                ThesaurusExtractDef thesaurusExtractDef = ThesaurusExtractDefBuilder.init(thesaurusFilter)
                        .setBoolean(ThesaurusExtractDef.WITHTHESAURUSTITLE_BOOLEAN, true)
                        .toThesaurusExtractDef();
                defaultExtractionDef = ExtractionDefBuilder.init()
                        .setDynamicThesaurusExtractDef(thesaurusExtractDef)
                        .toExtractionDef();
            } else {
                defaultExtractionDef = null;
            }
        }
        return defaultExtractionDef;
    }

    public static Transformer newTransformer(BdfServer bdfServer, PathConfiguration pathConfiguration, Templates templates) {
        try {
            Transformer transformer = templates.newTransformer();
            transformer.setURIResolver(wrap(BdfURI.getURIResolver(bdfServer, pathConfiguration)));
            return transformer;
        } catch (TransformerException e) {
            throw new NestedTransformerException(e);
        }
    }

    public static URIResolver wrap(URIResolver uriResolver) {
        return new InternalURIResolver(uriResolver);
    }

    public static TransformerFactory newTransformationFactory(BdfServer bdfServer, PathConfiguration pathConfiguration) {
        TransformerFactory factory = TransformerFactory.newInstance();
        factory.setURIResolver(wrap(BdfURI.getURIResolver(bdfServer, pathConfiguration)));
        return factory;
    }

    public static ExtensionInfo[] getExtensionInfoArray(BdfServer bdfServer, TransformationCheck transformationCheck) {
        SortedMap<String, ExtensionInfo> resultMap = new TreeMap<String, ExtensionInfo>();
        Set<String> withDefaultSet = bdfServer.getTransformationManager().getDefaultStreamTemplateAvailableExtensionSet(transformationCheck.getTransformationKey());
        for (String extension : withDefaultSet) {
            TemplateDescription[] templateDescriptionArray = transformationCheck.getStreamTemplateDescriptionArray(extension);
            if (templateDescriptionArray == null) {
                templateDescriptionArray = EMPTY_ARRAY;
            }
            resultMap.put(extension, new ExtensionInfo(extension, getAction(extension), LocalisationUtils.toMessage("_ link.global.version", extension.toUpperCase()), true, templateDescriptionArray));
        }
        for (String availableExtension : transformationCheck.getExtensionArray()) {
            if (!resultMap.containsKey(availableExtension)) {
                TemplateDescription[] templateDescriptionArray = transformationCheck.getStreamTemplateDescriptionArray(availableExtension);
                resultMap.put(availableExtension, new ExtensionInfo(availableExtension, getAction(availableExtension), LocalisationUtils.toMessage("_ link.global.version", availableExtension.toUpperCase()), false, templateDescriptionArray));
            }
        }
        return resultMap.values().toArray(new ExtensionInfo[resultMap.size()]);
    }


    public static boolean isUniquelang(BdfServer bdfServer) {
        LangConfiguration langConfiguration = bdfServer.getLangConfiguration();
        if (langConfiguration.isAllLanguages()) {
            return false;
        }
        int count = langConfiguration.getWorkingLangs().size();
        if ((count == 1) && (langConfiguration.getSupplementaryLangs().isEmpty())) {
            return true;
        } else {
            return false;
        }
    }

    public static List<FilterUnit> toFilterUnitList(Corpus corpus, UiComponents uiComponents, boolean addChrono, boolean addFichePhrase, Predicate<UiComponent> predicate) {
        return initFilterUnitListBuilder(corpus, uiComponents, addChrono, addFichePhrase, predicate).toFilterUnitList();
    }

    public static List<Corpus> getCorpusList(Fichotheque fichotheque, Attributes attributes) {
        Attribute corpusAttribute = attributes.getAttribute(TransformationSpace.CORPUS_KEY);
        if (corpusAttribute != null) {
            List<Corpus> list = new ArrayList<Corpus>();
            for (String value : corpusAttribute) {
                Corpus corpus = FichothequeUtils.getCorpus(fichotheque, value);
                if (corpus != null) {
                    list.add(corpus);
                }
            }
            if (!list.isEmpty()) {
                return list;
            }
        }
        return fichotheque.getCorpusList();
    }

    private static FilterUnitListBuilder initFilterUnitListBuilder(Corpus corpus, UiComponents uiComponents, boolean addChrono, boolean addFichePhrase, Predicate<UiComponent> predicate) {
        Fichotheque fichotheque = corpus.getFichotheque();
        FilterUnitListBuilder builder = new FilterUnitListBuilder();
        for (UiComponent uiComponent : uiComponents.getUiComponentList()) {
            if (!predicate.test(uiComponent)) {
                continue;
            }
            if (uiComponent instanceof FieldUi) {
                builder.addFieldKeyFilter(((FieldUi) uiComponent).getFieldKey(), null);
            } else if (uiComponent instanceof SpecialIncludeUi) {
                switch (uiComponent.getName()) {
                    case FichothequeConstants.DATECREATION_NAME:
                    case FichothequeConstants.DATEMODIFICATION_NAME:
                        builder.addChronoFilter();
                        break;
                    case FichothequeConstants.LIAGE_NAME:
                        builder.addCorpusExtractFilter(getLiageCorpusExtractDef(fichotheque, (SpecialIncludeUi) uiComponent, UiUtils.checkLiageTest(uiComponents)));
                        break;
                    case FichothequeConstants.RATTACHEMENT_NAME:
                        Subset masterSubset = corpus.getMasterSubset();
                        if ((masterSubset != null) && (masterSubset instanceof Thesaurus)) {
                            MotcleFilter motcleFilter = MotcleFilterBuilder.init(MotcleFilter.DEFAULT_TYPE)
                                    .setWithLabels(true)
                                    .setWithFicheStylePhrase(((Thesaurus) masterSubset).getThesaurusMetadata().isWithFicheStyle())
                                    .toMotcleFilter();
                            builder.addMotcleRattachementFilter(getMotcleRattachementDef(motcleFilter));
                        }
                        builder.addFicheRattachementFilter(getFicheRattachementDef(ExtractionUtils.TITRE_FICHEFILTER));
                        break;
                }
            } else if (uiComponent instanceof SubsetIncludeUi) {
                SubsetIncludeUi subsetIncludeUi = (SubsetIncludeUi) uiComponent;
                builder.addDefaultFilter(subsetIncludeUi.getExtendedIncludeKey(), fichotheque);
            }
        }
        if (addChrono) {
            builder.addChronoFilter();
        }
        if (addFichePhrase) {
            builder.addPhraseFilter(FichothequeConstants.FICHE_PHRASE);
        }
        return builder;
    }

    public static ExtractParameters buildExtractParameters(ExtractionContext extractionContext, Attributes templateAttributes, @Nullable BdfParameters bdfParameters) {
        int extractVersion = TransformationUtils.getExtractVersion(templateAttributes);
        boolean userOverride = false;
        boolean withEmpty = false;
        if (bdfParameters != null) {
            BdfUser bdfUser = bdfParameters.getBdfUser();
            if (bdfUser != null) {
                boolean hideEmpty = BdfUserUtils.getBoolean(bdfUser, BdfUserSpace.HIDEEMPTY_KEY);
                if (hideEmpty) {
                    userOverride = true;
                    withEmpty = false;
                }
            }
        }
        if (!userOverride) {
            withEmpty = TransformationUtils.isWithEmptyComponents(templateAttributes, false);
        }
        return new ExtractParameters(extractionContext, extractVersion, withEmpty);
    }


    private static ExtractionDef getCompilationExtractionDef(BdfServer bdfServer, Attributes attributes) {
        ExtractionDefBuilder extractionDefBuilder = new ExtractionDefBuilder();
        Fichotheque fichotheque = bdfServer.getFichotheque();
        UiManager uiManager = bdfServer.getUiManager();
        for (Corpus corpus : getCorpusList(fichotheque, attributes)) {
            UiComponents uiComponents = uiManager.getMainUiComponents(corpus);
            List<FilterUnit> filterUnitList = toFilterUnitList(corpus, uiComponents, true, true, UiUtils.ALL_COMPONENTPREDICATE);
            FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(filterUnitList);
            FicheQuery ficheQuery = FicheQueryBuilder.init()
                    .addCorpus(corpus)
                    .toFicheQuery();
            CorpusExtractDef corpusExtractDef = CorpusExtractDefBuilder.init(ficheFilter)
                    .setClause(CorpusExtractDef.TITLE_CLAUSE, TitleClauseBuilder.DEFAULT)
                    .setEntryList(SelectionUtils.toFicheConditionEntry(ficheQuery))
                    .toCorpusExtractDef();
            extractionDefBuilder.addStatic(corpusExtractDef);
        }
        return extractionDefBuilder.toExtractionDef();
    }

    private static ExtractionDef getFicheExtractionDef(BdfServer bdfServer, SubsetKey corpusKey, Attributes attributes) {
        Corpus corpus = (Corpus) bdfServer.getFichotheque().getSubset(corpusKey);
        UiComponents uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        List<Corpus> includedCorpusList = BdfServerUtils.getIncludeSatelliteList(corpus);
        Predicate<UiComponent> predicate;
        if (!includedCorpusList.isEmpty()) {
            predicate = UiUtils.MAIN_COMPONENTPREDICATE;
        } else {
            predicate = UiUtils.ALL_COMPONENTPREDICATE;
        }
        FilterUnitListBuilder builder = initFilterUnitListBuilder(corpus, uiComponents, true, true, predicate);
        for (Corpus satellite : includedCorpusList) {
            List<FilterUnit> satelliteFilterUnitList = toFilterUnitList(satellite, bdfServer.getUiManager().getMainUiComponents(satellite), false, false, UiUtils.SATELLITE_COMPONENTPREDICATE);
            FicheFilter satelliteFicheFilter = FicheFilterBuilder.fromFilterUnitList(satelliteFilterUnitList);
            FicheRattachementDef ficheRattachementDef = FicheRattachementDefBuilder.init(satelliteFicheFilter).addRattachementSubsetKey(satellite.getSubsetKey()).toFicheRattachementDef();
            builder.addFicheRattachementFilter(ficheRattachementDef);
        }
        FicheFilter ficheFilter = FicheFilterBuilder.fromFilterUnitList(builder.toFilterUnitList());
        int extractVersion = TransformationUtils.getExtractVersion(attributes);
        TagNameInfo tagNameInfo;
        if (extractVersion == ExtractionConstants.INITIAL_VERSION) {
            tagNameInfo = TagNameInfo.DEFAULT;
        } else {
            tagNameInfo = TagNameInfo.NULL;
        }
        return ExtractionDefBuilder.init()
                .setDynamicCorpusExtractDef(CorpusExtractDefBuilder
                        .init(ficheFilter)
                        .setTagNameInfo(tagNameInfo)
                        .toCorpusExtractDef())
                .toExtractionDef();
    }

    private static CorpusExtractDef getLiageCorpusExtractDef(Fichotheque fichotheque, SpecialIncludeUi specialIncludeUi, LiageTest liageTest) {
        CorpusExtractDefBuilder builder = CorpusExtractDefBuilder.init(ExtractionUtils.TITRE_FICHEFILTER)
                .setName(FichothequeConstants.LIAGE_NAME);
        LimitOption limitOption = specialIncludeUi.getLimitOption();
        if (limitOption.isNoNewAllowed()) {
            builder.setBoolean(CorpusExtractDef.HIDEEMPTY_BOOLEAN, true);
        }
        List<FicheCondition.Entry> entryList = new ArrayList<FicheCondition.Entry>();
        List<SubsetKey> excludeList = new ArrayList<SubsetKey>();
        for (Corpus otherCorpus : fichotheque.getCorpusList()) {
            SubsetKey otherKey = otherCorpus.getSubsetKey();
            Object testResult = liageTest.ownsToLiage(otherKey);
            if (testResult instanceof Ranges) {
                Ranges excludeRanges = (Ranges) testResult;
                FicheQuery ficheQuery = FicheQueryBuilder.init()
                        .addCorpus(otherKey)
                        .toFicheQuery();
                CroisementCondition croisementCondition = SelectionUtils.toCroisementCondition("", excludeRanges, true);
                entryList.add(SelectionUtils.toFicheConditionEntry(ficheQuery, croisementCondition));
                excludeList.add(otherKey);
            } else {
                Boolean bool = (Boolean) testResult;
                if (!bool) {
                    excludeList.add(otherKey);
                }
            }
        }
        FicheQueryBuilder globalBuilder = FicheQueryBuilder.init();
        if (!excludeList.isEmpty()) {
            globalBuilder
                    .setExclude(true)
                    .addCorpus(excludeList);
        }
        entryList.add(SelectionUtils.toFicheConditionEntry(globalBuilder.toFicheQuery(), SelectionUtils.toCroisementCondition("", null, true)));
        builder.setEntryList(entryList);
        return builder.toCorpusExtractDef();
    }

    private static FicheRattachementDef getFicheRattachementDef(FicheFilter ficheFilter) {
        return FicheRattachementDefBuilder.init(ficheFilter)
                .toFicheRattachementDef();
    }

    private static MotcleRattachementDef getMotcleRattachementDef(MotcleFilter motcleFilter) {
        return MotcleRattachementDefBuilder.init(motcleFilter)
                .toMotcleRattachementDef();
    }

    public static String getShouldNotOccurMessage(TemplateCompiler templateCompiler) {
        List<TemplateContentDescription> list = templateCompiler.flushTemplateContentDescriptionList();
        StringBuilder buf = new StringBuilder();
        for (TemplateContentDescription description : list) {
            for (MessageByLine messageByLine : description.getMessageByLineList()) {
                for (MessageByLine.Category category : messageByLine.getCategoryList()) {
                    if (buf.length() > 0) {
                        buf.append(" @@@ ");
                    }
                    buf.append(category.getName());
                    for (Message message : category.getMessageList()) {
                        buf.append(" | ");
                        buf.append(message.getMessageKey());
                        Object[] values = message.getMessageValues();
                        for (Object value : values) {
                            buf.append(" / ");
                            buf.append(value);
                        }
                    }
                }
            }
        }
        if (buf.length() == 0) {
            buf.append("Unknown compilation error");
        }
        return buf.toString();
    }


    private static class InternalURIResolver implements URIResolver {

        private final URIResolver originalUriResolver;

        private InternalURIResolver(URIResolver originalUriResolver) {
            this.originalUriResolver = originalUriResolver;
        }

        @Override
        public Source resolve(String href, String base) throws TransformerException {
            Source source = originalUriResolver.resolve(href, base);
            if (source == null) {
                throw new TransformerException("(base: " + base + ")");
            } else {
                return source;
            }
        }

    }

    private static String getAction(String extension) {
        switch (extension) {
            case "ods":
                return "action-Ods";
            case "odt":
                return "action-Odt";
            default:
                return "action-Test";
        }
    }

}
