/* BdfServer_Get - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.instructions.tables;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.exportation.table.TableExportParameters;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.interaction.GetConstants;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.get.streamproducers.TableExportOdsProducer;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.CellEngineUtils;
import fr.exemole.bdfserver.tools.exportation.table.TableExportParametersBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.interaction.FicheTableParametersParser;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.exportation.table.TableExport;
import net.fichotheque.exportation.table.TableExportContext;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.utils.TableExportUtils;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.ServletUtils;


/**
 *
 * @author Vincent Calame
 */
public class MultiTableExportEngine {

    private final static DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
    private final BdfParameters bdfParameters;
    private final TableExportParameters tableExportParameters;
    private final boolean merge;
    private final boolean withThesaurusTable;

    private MultiTableExportEngine(BdfParameters bdfParameters, TableExportParameters tableExportParameters, boolean merge, boolean withThesaurusTable) {
        this.bdfParameters = bdfParameters;
        this.tableExportParameters = tableExportParameters;
        this.merge = merge;
        this.withThesaurusTable = withThesaurusTable;
    }

    public ResponseHandler run() {
        PermissionSummary permissionSummary = bdfParameters.getPermissionSummary();
        BdfServer bdfServer = bdfParameters.getBdfServer();
        TableExportManager tableExportManager = bdfServer.getTableExportManager();
        StringBuilder buf = new StringBuilder();
        buf.append("tableexport-");
        buf.append(bdfServer.getFichotheque().getFichothequeMetadata().getBaseName());
        TableExport tableExport = null;
        String tableExportName = tableExportParameters.getTableExportName();
        if (tableExportName != null) {
            if (tableExportManager.containsTableExport(tableExportName)) {
                buf.append("-");
                buf.append(tableExportName);
            } else {
                tableExportName = null;
            }
        }
        if (tableExportName == null) {
            tableExport = BdfTableExportUtils.toDefaultTableExport(bdfServer, tableExportParameters.getDefaulFicheTableParameters(), permissionSummary);
        } else {
            tableExport = tableExportManager.getTableExport(tableExportName);
        }
        buf.append(".");
        buf.append(dateFormat.format(new Date()));
        buf.append(".ods");
        TableExportContext tableExportContext = bdfServer.getTableExportContext();
        ExtractionContext extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfParameters)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        CellEngine cellEngine = CellEngineUtils.newCellEngine(bdfServer, extractionContext, tableExport, false);
        StreamProducer streamProducer = TableExportOdsProducer.newMultiInstance(buf.toString(), bdfParameters, tableExport, bdfParameters.getBdfUser().getSelectedFiches(), tableExportParameters.getHeaderType(), merge, withThesaurusTable, tableExportContext, cellEngine);
        return ServletUtils.wrap(streamProducer);
    }

    public static MultiTableExportEngine buildFromRequest(BdfParameters bdfParameters, RequestMap requestMap) throws ErrorMessageException {
        TableExportParameters tableExportParameters = getTableExportParameters(bdfParameters.getBdfServer(), requestMap);
        boolean merge = requestMap.isTrue(GetConstants.MERGE_PARAMNAME);
        boolean withThesaurusTable = requestMap.isTrue(GetConstants.THESAURUSTABLE_PARAMNAME);
        return new MultiTableExportEngine(bdfParameters, tableExportParameters, merge, withThesaurusTable);
    }

    private static TableExportParameters getTableExportParameters(BdfServer bdfServer, RequestMap requestMap) throws ErrorMessageException {
        FicheTableParameters ficheTableParameters = FicheTableParametersParser.parse(requestMap);
        TableExportParametersBuilder tableExportParametersBuilder = new TableExportParametersBuilder();
        tableExportParametersBuilder.setFicheTableParameters(ficheTableParameters);
        String tableExportName = null;
        String defaultExportParam = requestMap.getParameter(GetConstants.DEFAULTEXPORT_PARAMNAME);
        if ((defaultExportParam != null) && (defaultExportParam.equals("0"))) {
            tableExportName = requestMap.getParameter(InteractionConstants.TABLEEXPORT_PARAMNAME);
            if (tableExportName != null) {
                boolean present = bdfServer.getTableExportManager().containsTableExport(tableExportName);
                if (!present) {
                    throw BdfErrors.unknownParameterValue(InteractionConstants.TABLEEXPORT_PARAMNAME, tableExportName);
                }
            }
        }
        tableExportParametersBuilder.setTableExportName(tableExportName);
        String headerTypeValue = requestMap.getParameter(InteractionConstants.HEADERTYPE_PARAMNAME);
        if (headerTypeValue == null) {
            throw BdfErrors.emptyMandatoryParameter(InteractionConstants.HEADERTYPE_PARAMNAME);
        }
        try {
            short headerType = TableExportUtils.headerTypeToShort(headerTypeValue);
            tableExportParametersBuilder.setHeaderType(headerType);
        } catch (IllegalArgumentException iae) {
            throw BdfErrors.wrongParameterValue(InteractionConstants.HEADERTYPE_PARAMNAME, headerTypeValue);
        }
        return tableExportParametersBuilder.toTableExportParameters();
    }

}
