/* FichothequeLib_API - Copyright (c) 2019-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.util.Collections;
import java.util.List;
import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.LinkAnalyser;
import net.fichotheque.extraction.ModeResolver;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.run.CorpusExtractResult;
import net.fichotheque.extraction.run.ExtractorProvider;
import net.fichotheque.extraction.run.ThesaurusExtractResult;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.MessageLocalisationProvider;
import net.mapeadores.util.mimetype.MimeTypeResolver;


/**
 *
 * @author Vincent Calame
 */
public final class ExtractionUtils {

    public final static FicheFilter TITRE_FICHEFILTER = new TitreFicheFilter();
    public final static List<CorpusExtractDef> EMPTY_CORPUSEXTRACTDEFLIST = Collections.emptyList();
    public final static List<FicheRattachementDef> EMPTY_FICHERATTACHEMENTDEFLIST = Collections.emptyList();
    public final static MotcleFilter LABELS_MOTCLEFILTER = new DefaultMotcleFilter(true);
    public final static MotcleFilter NONE_MOTCLEFILTER = new DefaultMotcleFilter(false);
    public final static IllustrationFilter DEFAULT_ILLUSTRATIONFILTER = new DefaultIllustrationFilter();
    public final static DocumentFilter DEFAULT_DOCUMENTFILTER = new DefaultDocumentFilter();
    public final static List<CorpusExtractResult> EMPTY_CORPUSEXTRACTRESULTLIST = Collections.emptyList();
    public final static List<ThesaurusExtractResult> EMPTY_THESAURUSEXTRACTRESULTLIST = Collections.emptyList();

    private ExtractionUtils() {

    }

    public static int parseExtractVersion(String text) {
        if (text == null) {
            return ExtractionConstants.INITIAL_VERSION;
        }
        try {
            int value = Integer.parseInt(text);
            switch (value) {
                case ExtractionConstants.INITIAL_VERSION:
                    return ExtractionConstants.INITIAL_VERSION;
                default:
                    return ExtractionConstants.PLURAL_VERSION;
            }
        } catch (NumberFormatException nfe) {
            return ExtractionConstants.PLURAL_VERSION;
        }
    }

    public static String checkSort(String sort) {
        if ((sort == null) || (sort.length() == 0)) {
            return ExtractionConstants.ASCENDING_SORT;
        }
        switch (sort) {
            case ExtractionConstants.DESCENDING_SORT:
                return ExtractionConstants.DESCENDING_SORT;
            default:
                return ExtractionConstants.ASCENDING_SORT;
        }
    }

    public static String checkType(String type) {
        if ((type == null) || (type.length() == 0)) {
            throw new IllegalArgumentException("undefined type");
        }
        switch (type) {
            case "poids":
                return ExtractionConstants.POIDS_TYPE;
            case "titre":
                return ExtractionConstants.TITRE_TYPE;
            case "annee":
                return ExtractionConstants.ANNEE_TYPE;
            case "lang":
                return ExtractionConstants.LANG_TYPE;
            case "pays":
                return ExtractionConstants.PAYS_TYPE;
            case "fields":
                return ExtractionConstants.FIELDS_TYPE;
            case "position":
                return ExtractionConstants.POSITION_TYPE;
            default:
                throw new IllegalArgumentException("unknwon type : " + type);
        }
    }

    public static ExtractionContext derive(ExtractionContext extractionContext, Predicate<FicheMeta> fichePredicate) {
        return new DeriveExtractionContext(extractionContext, fichePredicate);
    }

    public static ExtractionContext derive(ExtractionContext extractionContext, LangContext langContext) {
        return new DeriveExtractionContext(extractionContext, langContext);
    }


    private static class DeriveExtractionContext implements ExtractionContext {

        private final ExtractionContext originExtractionContext;
        private final Predicate<FicheMeta> fichePredicate;
        private final LangContext langContext;

        private DeriveExtractionContext(ExtractionContext originExtractionContext, Predicate<FicheMeta> fichePredicate) {
            this.originExtractionContext = originExtractionContext;
            this.langContext = originExtractionContext.getLangContext();
            this.fichePredicate = fichePredicate;
        }

        private DeriveExtractionContext(ExtractionContext originExtractionContext, LangContext langContext) {
            this.originExtractionContext = originExtractionContext;
            this.langContext = langContext;
            this.fichePredicate = originExtractionContext.getFichePredicate();
        }

        @Override
        public Fichotheque getFichotheque() {
            return originExtractionContext.getFichotheque();
        }

        @Override
        public MessageLocalisationProvider getMessageLocalisationProvider() {
            return originExtractionContext.getMessageLocalisationProvider();
        }

        @Override
        public LangContext getLangContext() {
            return langContext;
        }

        @Override
        public LinkAnalyser getLinkAnalyser() {
            return originExtractionContext.getLinkAnalyser();
        }

        @Override
        public ExtractorProvider getExtractorProvider() {
            return originExtractionContext.getExtractorProvider();
        }

        @Override
        public MimeTypeResolver getMimeTypeResolver() {
            return originExtractionContext.getMimeTypeResolver();
        }

        @Override
        public ModeResolver getModeResolver() {
            return originExtractionContext.getModeResolver();
        }

        @Override
        public Predicate<Subset> getSubsetAccessPredicate() {
            return originExtractionContext.getSubsetAccessPredicate();
        }

        @Override
        public Predicate<FicheMeta> getFichePredicate() {
            return fichePredicate;
        }

        @Override
        public Fiches getSelectedFiches() {
            return originExtractionContext.getSelectedFiches();
        }

    }


    private static class TitreFicheFilter implements FicheFilter {

        private final List<FilterUnit> filterUnitList;

        private TitreFicheFilter() {
            FilterUnit[] array = new FilterUnit[2];
            array[0] = FilterUnitUtils.toFieldKeyFilterUnit(FieldKey.TITRE, null);
            array[1] = FilterUnitUtils.FICHEPHRASE_FILTERUNIT;
            filterUnitList = FilterUnitUtils.wrap(array);
        }

        @Override
        public boolean isWithCorpsdefiche() {
            return false;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

        @Override
        public TagNameInfo getTagNameInfo() {
            return TagNameInfo.DEFAULT;
        }

    }


    private static class DefaultMotcleFilter implements MotcleFilter {

        private final boolean withLabels;

        private DefaultMotcleFilter(boolean withLabels) {
            this.withLabels = withLabels;
        }

        @Override
        public short getType() {
            return MotcleFilter.DEFAULT_TYPE;
        }


        @Override
        public TagNameInfo getTagNameInfo() {
            return TagNameInfo.DEFAULT;
        }

        @Override
        public boolean isNoneFiltering() {
            return !withLabels;
        }

        @Override
        public boolean withLevel() {
            return false;
        }

        @Override
        public boolean withLabels() {
            return withLabels;
        }

        @Override
        public boolean withFicheStylePhrase() {
            return false;
        }

        @Override
        public MotcleFilter getChildrenFilter() {
            return null;
        }

        @Override
        public MotcleFilter getParentFilter() {
            return null;
        }

        @Override
        public MotcleFilter getNextFilter() {
            return null;
        }

        @Override
        public MotcleFilter getPreviousFilter() {
            return null;
        }

        @Override
        public List<CorpusExtractDef> getIndexationCorpusExtractDefList() {
            return EMPTY_CORPUSEXTRACTDEFLIST;
        }

        @Override
        public List<FicheRattachementDef> getFicheRattachementDefList() {
            return EMPTY_FICHERATTACHEMENTDEFLIST;
        }


    }


    private static class DefaultDocumentFilter implements DocumentFilter {

        private DefaultDocumentFilter() {

        }

        @Override
        public TagNameInfo getTagNameInfo() {
            return TagNameInfo.DEFAULT;
        }

    }


    private static class DefaultIllustrationFilter implements IllustrationFilter {


        private DefaultIllustrationFilter() {

        }

        @Override
        public TagNameInfo getTagNameInfo() {
            return TagNameInfo.DEFAULT;
        }

    }

}
