/* FichothequeLib_API - Copyright (c) 2010-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import net.fichotheque.Metadata;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheCondition;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.selection.SubsetCondition;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.conditions.ConditionsXMLStorage;
import net.mapeadores.util.conditions.TextTest;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.XMLWriter;


/**
 *
 * @author Vincent Calame
 */
public final class FichothequeXMLUtils {

    private FichothequeXMLUtils() {
    }

    public static void write(Metadata metadata, XMLWriter writer) throws IOException {
        LabelUtils.addLabels(writer, metadata.getTitleLabels());
        LabelUtils.addPhrases(writer, metadata.getPhrases());
        AttributeUtils.addAttributes(writer, metadata.getAttributes());
    }

    public static void writeFicheQuery(XMLWriter xmlWriter, FicheQuery ficheQuery, @Nullable CroisementCondition croisementCondition) throws IOException {
        SubsetCondition corpusCondition = ficheQuery.getCorpusCondition();
        if (corpusCondition.isExclude()) {
            xmlWriter.addEmptyElement("exclude");
        }
        for (SubsetKey subsetKey : corpusCondition.getSubsetKeySet()) {
            xmlWriter.addSimpleElement("corpus", subsetKey.getSubsetName());
        }
        if (corpusCondition.isWithCurrent()) {
            xmlWriter.addEmptyElement("current");
        }
        Ranges idRanges = ficheQuery.getIdRanges();
        if (idRanges != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(idRanges));
        }
        String discardFilter = ficheQuery.getDiscardFilter();
        if (!discardFilter.equals(FicheQuery.ALL_DISCARDFILTER)) {
            xmlWriter.startOpenTag("discard");
            xmlWriter.addAttribute("filter", discardFilter);
            xmlWriter.closeEmptyTag();
        }
        writePeriodCondition(xmlWriter, ficheQuery.getPeriodCondition());
        FieldQuery fieldQuery = ficheQuery.getFieldQuery();
        if (fieldQuery != null) {
            short fieldConditionScope = fieldQuery.getScope();
            xmlWriter.startOpenTag("field-query");
            xmlWriter.addAttribute("scope", SelectionUtils.fieldConditionToString(fieldConditionScope));
            xmlWriter.endOpenTag();
            if (fieldConditionScope == FieldQuery.SELECTION_SCOPE) {
                for (FieldKey fieldKey : fieldQuery.getFieldKeyList()) {
                    xmlWriter.addSimpleElement("field", fieldKey.getFieldString());
                }
            }
            ConditionsXMLStorage.appendCondition(xmlWriter, fieldQuery.getContentCondition());
            xmlWriter.closeTag("field-query");
        }
        String redacteurFilter = ficheQuery.getRedacteurFilter();
        if (redacteurFilter.equals(FicheQuery.SOME_REDACTEURFILTER)) {
            List<String> redacteurGlobalIdList = ficheQuery.getRedacteurGlobalIdList();
            List<SubsetKey> sphereKeyList = ficheQuery.getSphereKeyList();
            if ((!redacteurGlobalIdList.isEmpty()) || (!sphereKeyList.isEmpty())) {
                xmlWriter.openTag("redacteurs");
                for (String globalId : redacteurGlobalIdList) {
                    xmlWriter.startOpenTag("redacteur");
                    try {
                        xmlWriter.addAttribute("sphere", SphereUtils.getSubsetKey(globalId).getSubsetName());
                        xmlWriter.addAttribute("id", String.valueOf(SphereUtils.getId(globalId)));
                    } catch (ParseException pe) {
                    }
                    xmlWriter.closeEmptyTag();
                }
                for (SubsetKey subsetKey : sphereKeyList) {
                    xmlWriter.addSimpleElement("sphere", subsetKey.getSubsetName());
                }
                xmlWriter.closeTag("redacteurs");
            }
        } else if (!redacteurFilter.equals(FicheQuery.ALL_REDACTEURFILTER)) {
            xmlWriter.startOpenTag("redacteurs");
            xmlWriter.addAttribute("filter", redacteurFilter);
            xmlWriter.closeEmptyTag();
        }
        if (ficheQuery.isWithGeoloc()) {
            xmlWriter.addEmptyElement("geoloc");
        }
        MotcleCondition motcleCondition = ficheQuery.getMotcleCondition();
        if (motcleCondition != null) {
            List<MotcleCondition.Entry> indexationList = motcleCondition.getEntryList();
            if (indexationList.size() > 1) {
                xmlWriter.startOpenTag("motcle-query-mode");
                xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(motcleCondition.getSelectMode()));
                xmlWriter.closeEmptyTag();
            }
            for (MotcleCondition.Entry indexationEntry : indexationList) {
                xmlWriter.openTag("motcle-query");
                writeMotcleQuery(xmlWriter, indexationEntry.getMotcleQuery(), indexationEntry.getCroisementCondition());
                xmlWriter.closeTag("motcle-query");
            }
        }
        if (ficheQuery.isWithRattachement()) {
            xmlWriter.addEmptyElement("rattachement");
        }
        if (croisementCondition != null) {
            writeCroisementCondition(xmlWriter, croisementCondition);
        }
    }

    public static void writeMotcleQuery(XMLWriter xmlWriter, MotcleQuery motcleQuery, @Nullable CroisementCondition croisementCondition) throws IOException {
        for (SubsetKey subsetKey : motcleQuery.getThesaurusKeyList()) {
            xmlWriter.addSimpleElement("thesaurus", subsetKey.getSubsetName());
        }
        Ranges idRanges = motcleQuery.getIdRanges();
        if (idRanges != null) {
            xmlWriter.addSimpleElement("range", RangeUtils.positiveRangetoString(idRanges));
        }
        Ranges levelRanges = motcleQuery.getLevelRanges();
        if (levelRanges != null) {
            xmlWriter.addSimpleElement("level", RangeUtils.positiveRangetoString(levelRanges));
        }
        Condition contentCondition = motcleQuery.getContentCondition();
        if (contentCondition != null) {
            short scope = motcleQuery.getContentConditionScope();
            xmlWriter.startOpenTag("condition");
            xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(contentCondition.getMode()));
            if (scope != MotcleQuery.ONLY_IDALPHA_SCOPE) {
                xmlWriter.addAttribute("scope", SelectionUtils.motcleFieldConditionScopeToString(scope));
            }
            xmlWriter.endOpenTag();
            for (TextTest textTest : contentCondition.getIncludingTextTestList()) {
                ConditionsXMLStorage.appendTestText(textTest, xmlWriter);
            }
            for (TextTest textTest : contentCondition.getExcludingTextTestList()) {
                ConditionsXMLStorage.appendTestText(textTest, xmlWriter);
            }
            xmlWriter.closeTag("condition");
        }
        FicheCondition ficheCondition = motcleQuery.getFicheCondition();
        if (ficheCondition != null) {
            List<FicheCondition.Entry> indexationList = ficheCondition.getEntryList();
            if (indexationList.size() > 1) {
                xmlWriter.startOpenTag("fiche-query-mode");
                xmlWriter.addAttribute("mode", ConditionsUtils.modeToString(ficheCondition.getSelectMode()));
                xmlWriter.closeEmptyTag();
            }
            for (FicheCondition.Entry ficheEntry : indexationList) {
                xmlWriter.openTag("fiche-query");
                writeFicheQuery(xmlWriter, ficheEntry.getFicheQuery(), ficheEntry.getCroisementCondition());
                xmlWriter.closeTag("fiche-query");
            }
        }
        if (motcleQuery.includeSatellites()) {
            xmlWriter.addEmptyElement("satellites");
        }
        if (croisementCondition != null) {
            writeCroisementCondition(xmlWriter, croisementCondition);
        }
    }

    public static void writeIllustrationQuery(XMLWriter xmlWriter, IllustrationQuery illustrationQuery, @Nullable CroisementCondition croisementCondition) throws IOException {
        for (SubsetKey subsetKey : illustrationQuery.getAlbumKeyList()) {
            xmlWriter.addSimpleElement("album", subsetKey.getSubsetName());
        }
        for (String albumDimName : illustrationQuery.getAlbumDimNameList()) {
            xmlWriter.addSimpleElement("albumdim", albumDimName);
        }
        if (croisementCondition != null) {
            writeCroisementCondition(xmlWriter, croisementCondition);
        }
    }


    public static void writeDocumentQuery(XMLWriter xmlWriter, DocumentQuery documentQuery, @Nullable CroisementCondition croisementCondition) throws IOException {
        for (SubsetKey subsetKey : documentQuery.getAddendaKeyList()) {
            xmlWriter.addSimpleElement("addenda", subsetKey.getSubsetName());
        }
        if (croisementCondition != null) {
            writeCroisementCondition(xmlWriter, croisementCondition);
        }
    }

    private static void writeCroisementCondition(XMLWriter xmlWriter, CroisementCondition croisementCondition) throws IOException {
        if (croisementCondition != null) {
            String[] lienModeArray = croisementCondition.getLienModeArray();
            if (lienModeArray != null) {
                int length = lienModeArray.length;
                for (int i = 0; i < length; i++) {
                    String mode = lienModeArray[i];
                    if (mode.isEmpty()) {
                        mode = "_default";
                    }
                    xmlWriter.addSimpleElement("lien-mode", mode);
                }
            }
            Ranges poidsRanges = croisementCondition.getPoidsRanges();
            if (poidsRanges != null) {
                xmlWriter.startOpenTag("lien-poids");
                if (croisementCondition.isExclusionRangeList()) {
                    xmlWriter.addAttribute("mode", "exclude");
                }
                xmlWriter.endOpenTag();
                xmlWriter.addText(RangeUtils.positiveRangetoString(poidsRanges));
                xmlWriter.closeTag("lien-poids", false);
            }
        }
    }

    private static void writePeriodCondition(XMLWriter xmlWriter, PeriodCondition periodCondition) throws IOException {
        if (periodCondition == null) {
            return;
        }
        xmlWriter.startOpenTag("period");
        if (periodCondition.getStartType().equals(PeriodCondition.ANY_TYPE)) {
            xmlWriter.addAttribute("start", PeriodCondition.ANY_CHAR);
        } else {
            xmlWriter.addAttribute("start", periodCondition.getStartDate().toString());
        }
        switch (periodCondition.getEndType()) {
            case PeriodCondition.ANY_TYPE:
                xmlWriter.addAttribute("end", PeriodCondition.ANY_CHAR);
                break;
            case PeriodCondition.DATE_TYPE:
                xmlWriter.addAttribute("end", periodCondition.getEndDate().toString());
                break;
        }
        xmlWriter.endOpenTag();
        if (periodCondition.isOnCreationDate()) {
            xmlWriter.addSimpleElement("chrono", "creation");
        }
        if (periodCondition.isOnModificationDate()) {
            xmlWriter.addSimpleElement("chrono", "modification");
        }
        for (FieldKey fieldKey : periodCondition.getFieldKeyList()) {
            xmlWriter.addSimpleElement("field", fieldKey.getFieldString());
        }
        xmlWriter.closeTag("period");
    }


}
